module AMS::Cursor
  class << self

    # Show/Hide cursor.
    # @param [Boolean] state
    # @return [Boolean] Whether cursor visibility state was changed.
    # @since 2.0.0
    def show(state)
      AMS::C.set_cursor_visible(state)
    end

    # Determine whether cursor is visible.
    # @return [Boolean]
    # @since 2.0.0
    def is_visible?
      AMS::C.is_cursor_visible?()
    end

    # Get cursor position.
    # @param [Fixnum] mode
    #   +1+ : Retrieve coordinates relative to screen origin.
    #   +2+ : Retrieve coordinates relative to the viewport origin.
    # @return [Array<Fixnum>] +[x,y]+
    # @since 2.0.0
    def get_pos(mode = 1)
      x,y = AMS::C.get_cursor_pos()
      if mode == 2
        srect = AMS::C.get_viewport_rect()
        x -= srect[0]
        y -= srect[1]
      end
      return [x,y]
    end

    # Set cursor position.
    # @param [Fixnum] x
    # @param [Fixnum] y
    # @param [Fixnum] mode
    #   +1+ : Given coordinates are relative to screen origin.
    #   +2+ : Given coordinates are relative to the viewport origin.
    # @return [Boolean] success
    # @since 2.0.0
    def set_pos(x, y, mode = 1)
      if mode == 2
        srect = AMS::C.get_viewport_rect()
        x += srect[0]
        y += srect[1]
      end
      AMS::C.set_cursor_pos(x,y)
    end

    # Get upper-left and lower-right coordinates of the cursor clip rectangle.
    # @param [Fixnum] mode
    #   +1+ : Retrieve coordinates relative to screen origin.
    #   +2+ : Retrieve coordinates relative to the viewport origin.
    # @return [Array<Fixnum>] +[x1,y1, x2,y2]+
    # @since 2.0.0
    def get_clip_rect(mode = 1)
      rect = AMS::C.get_cursor_clip_rect()
      if mode == 2
        srect = AMS::C.get_viewport_rect()
        rect[0] -= srect[0]
        rect[1] -= srect[1]
        rect[2] -= srect[0]
        rect[3] -= srect[1]
      end
      rect
    end

    # Set upper-left and lower-right coordinates of the cursor clip rectangle.
    # @param [Fixnum] x1 X coordinate of the upper-left corner of the rect.
    # @param [Fixnum] y1 Y coordinate of the upper-left corner of the rect.
    # @param [Fixnum] x2 X coordinate of the lower-right corner of the rect.
    # @param [Fixnum] y2 Y coordinate of the lower-right corner of the rect.
    # @param [Fixnum] mode
    #   +1+ : Given coordinates are relative to screen origin.
    #   +2+ : Given coordinates are relative to the viewport origin.
    # @return [Boolean] success
    # @since 2.0.0
    def set_clip_rect(x1, y1, x2, y2, mode = 1)
      rect = AMS::C.get_cursor_clip_rect()
      if mode == 2
        srect = AMS::C.get_viewport_rect()
        x1 += srect[0]
        y1 += srect[1]
        x2 += srect[0]
        y2 += srect[1]
      end
      AMS::C.set_cursor_clip_rect(x1, y1, x2, y2)
    end

    # Clip cursor to main window.
    # @return [Boolean] success
    # @since 2.0.0
    def clip_to_main_window
      AMS::C.set_cursor_clip_rect(*AMS::C.get_window_rect(AMS::C.get_main_window()))
    end

    # Clip cursor to view window.
    # @return [Boolean] success
    # @since 2.0.0
    def clip_to_viewport
      AMS::C.set_cursor_clip_rect(*AMS::C.get_viewport_rect())
    end

    # Unclip cursor.
    # @return [Boolean] success
    # @since 2.0.0
    def clear_clip
      AMS::C.clear_clip_cursor()
    end

    # Determine whether cursor is pointing at the main window.
    # @return [Boolean]
    # @since 2.0.0
    def is_main_window_target?
      x,y = AMS::C.get_cursor_pos()
      AMS::C.get_main_window() == AMS::C.get_window_from_point(x,y)
    end

    # Determine whether cursor is within the view client area.
    # @return [Boolean]
    # @since 2.0.0
    def is_viewport_target?
      rect = AMS::C.get_viewport_rect()
      x,y = AMS::C.get_cursor_pos()
      x.between?(rect[0], rect[2]) && y.between?(rect[1], rect[3])
    end

  end # class << self
end # module AMS::Cursor
