module AMS::Keyboard
  class << self

    # Get virtual key state
    # @param [String, Symbol, Fixnum] vk Virtual key name or constant value.
    # @return [Fixnum] +1+ if down or +0+ if up.
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx Virtual-Key Codes
    # @since 2.0.0
    def get_key_state(vk)
      unless vk.is_a?(Numeric)
        vk = AMS::C.get_virtual_key_value_by_name(vk)
        return 0 unless vk
      end
      AMS::C.get_key_state(vk.to_i)
    end

    alias key get_key_state

    # Get an array of all key states.
    # @return [Array<Fixnum>] An array of 256 values of +1+ (down) or +0+ (up).
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx Virtual-Key Codes
    # @since 2.0.0
    def get_keyboard_state
      AMS::C.get_keyboard_state()
    end

    # Get constant value associated with the virtual key name.
    # @param [String, Symbol] vk_name Virtual key name.
    # @return [Fixnum, nil] Virtual key constant value if successful.
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx Virtual-Key Codes
    # @since 2.0.0
    def get_key_value(vk_name)
      AMS::C.get_virtual_key_value_by_name(vk_name)
    end

    # Determine whether virtual key is toggled.
    # @param [String, Symbol, Fixnum] vk Virtual key name or constant value.
    # @return [Boolean]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx Virtual-Key Codes
    # @since 2.0.0
    def key_toggled?(vk)
      unless vk.is_a?(Numeric)
        vk = AMS::C.get_virtual_key_value_by_name(vk)
        return 0 unless vk
      end
      AMS::C.is_key_toggled?(vk.to_i)
    end

    # Determine whether virtual key is down.
    # @param [String, Symbol, Fixnum] vk Virtual key name or constant value.
    # @return [Boolean]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx Virtual-Key Codes
    # @since 2.0.0
    def key_down?(vk)
      get_key_state(vk) == 1
    end

    # Determine whether an array of virtual keys are down.
    # @param [Array<String, Symbol, Fixnum>] vks Virtual key name or constant value.
    # @return [Boolean]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx Virtual-Key Codes
    # @since 2.0.0
    def keys_down?(*vks)
      vks.flatten!
      vks.each { |vk|
        return false if get_key_state(vk) == 0
      }
      true
    end

    # Determine whether virtual key is up.
    # @param [String, Symbol, Fixnum] vk Virtual key name or constant value.
    # @return [Boolean]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx Virtual-Key Codes
    # @since 2.0.0
    def key_up?(vk)
      get_key_state(vk) == 0
    end

    # Determine whether an array of virtual keys are up.
    # @param [Array<String, Symbol, Fixnum>] vks Virtual key name or constant value.
    # @return [Boolean]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx Virtual-Key Codes
    # @since 2.0.0
    def keys_up?(*vks)
      vks.flatten!
      vks.each { |vk|
        return false if get_key_state(vk) == 1
      }
      true
    end

    # Determine whether ALT key is down.
    # @return [Boolean]
    # @since 2.0.0
    def menu_down?
      AMS::C.get_key_state(0x12) == 1
    end

    alias alt_down? menu_down?

    # Determine whether ALT key is up.
    # @return [Boolean]
    # @since 2.0.0
    def menu_up?
      AMS::C.get_key_state(0x12) == 0
    end

    alias alt_up? menu_up?

    # Determine whether CTRL key is down.
    # @return [Boolean]
    # @since 2.0.0
    def control_down?
      AMS::C.get_key_state(0x11) == 1
    end

    alias ctrl_down? control_down?

    # Determine whether CTRL key is up.
    # @return [Boolean]
    # @since 2.0.0
    def control_up?
      AMS::C.get_key_state(0x11) == 0
    end

    alias ctrl_up? control_up?

    # Determine whether SHIFT key is down.
    # @return [Boolean]
    # @since 2.0.0
    def shift_down?
      AMS::C.get_key_state(0x10) == 1
    end

    # Determine whether SHIFT key is up.
    # @return [Boolean]
    # @since 2.0.0
    def shift_up?
      AMS::C.get_key_state(0x10) == 0
    end

    # Determine whether left mouse button is down.
    # @return [Boolean]
    # @since 2.0.0
    def lbutton_down?
      AMS::C.get_key_state(0x01) == 1
    end

    # Determine whether left mouse button is up.
    # @return [Boolean]
    # @since 2.0.0
    def lbutton_up?
      AMS::C.get_key_state(0x01) == 0
    end

    # Determine whether right mouse button is down.
    # @return [Boolean]
    # @since 2.0.0
    def rbutton_down?
      AMS::C.get_key_state(0x02) == 1
    end

    # Determine whether right mouse button is up.
    # @return [Boolean]
    # @since 2.0.0
    def rbutton_up?
      AMS::C.get_key_state(0x02) == 0
    end

    # Determine whether middle mouse button is down.
    # @return [Boolean]
    # @since 2.0.0
    def mbutton_down?
      AMS::C.get_key_state(0x04) == 1
    end

    # Determine whether middle mouse button is up.
    # @return [Boolean]
    # @since 2.0.0
    def mbutton_up?
      AMS::C.get_key_state(0x04) == 0
    end

    # Determine whether X1 mouse button is down.
    # @return [Boolean]
    # @since 2.0.0
    def xbutton1_down?
      AMS::C.get_key_state(0x05) == 1
    end

    # Determine whether X1 mouse button is up.
    # @return [Boolean]
    # @since 2.0.0
    def xbutton1_up?
      AMS::C.get_key_state(0x05) == 0
    end

    # Determine whether X2 mouse button is down.
    # @return [Boolean]
    # @since 2.0.0
    def xbutton2_down?
      AMS::C.get_key_state(0x06) == 1
    end

    # Determine whether X2 mouse button is up.
    # @return [Boolean]
    # @since 2.0.0
    def xbutton2_up?
      AMS::C.get_key_state(0x06) == 0
    end

    # Determine whether left SHIFT key is down.
    # @return [Boolean]
    # @since 2.0.0
    def lshift_down?
      AMS::C.get_key_state(0xA0) == 1
    end

    # Determine whether left SHIFT key is up.
    # @return [Boolean]
    # @since 2.0.0
    def lshift_up?
      AMS::C.get_key_state(0xA0) == 0
    end

    # Determine whether right SHIFT key is down.
    # @return [Boolean]
    # @since 2.0.0
    def rshift_down?
      AMS::C.get_key_state(0xA1) == 1
    end

    # Determine whether right SHIFT key is up.
    # @return [Boolean]
    # @since 2.0.0
    def rshift_up?
      AMS::C.get_key_state(0xA1) == 0
    end

    # Determine whether left CTRL key is down.
    # @return [Boolean]
    # @since 2.0.0
    def lcontrol_down?
      AMS::C.get_key_state(0xA2) == 1
    end

    alias lctrl_down? lcontrol_down?

    # Determine whether left CTRL key is up.
    # @return [Boolean]
    # @since 2.0.0
    def lcontrol_up?
      AMS::C.get_key_state(0xA2) == 0
    end

    alias lctrl_up? lcontrol_up?

    # Determine whether right CTRL key is down.
    # @return [Boolean]
    # @since 2.0.0
    def rcontrol_down?
      AMS::C.get_key_state(0xA3) == 1
    end

    alias rctrl_down? rcontrol_down?

    # Determine whether right CTRL key is up.
    # @return [Boolean]
    # @since 2.0.0
    def rcontrol_up?
      AMS::C.get_key_state(0xA3) == 0
    end

    alias rctrl_up? rcontrol_up?

    # Determine whether left ALT key is down.
    # @return [Boolean]
    # @since 2.0.0
    def lmenu_down?
      AMS::C.get_key_state(0xA4) == 1
    end

    alias lalt_down? lmenu_down?

    # Determine whether left ALT key is up.
    # @return [Boolean]
    # @since 2.0.0
    def lmenu_up?
      AMS::C.get_key_state(0xA4) == 0
    end

    alias lalt_up? lmenu_up?

    # Determine whether right ALT key is down.
    # @return [Boolean]
    # @since 2.0.0
    def rmenu_down?
      AMS::C.get_key_state(0xA5) == 1
    end

    alias ralt_down? rmenu_down?

    # Determine whether right ALT key is up.
    # @return [Boolean]
    # @since 2.0.0
    def rmenu_up?
      AMS::C.get_key_state(0xA5) == 0
    end

    alias ralt_up? rmenu_up?

  end # class << self
end # module AMS::Keyboard
