module AMS::Menu
  class << self

    # Verify that handle is a reference to a valid menu.
    # @param [Fixnum] handle
    # @return [void]
    # @raise [TypeError] if handle is not a reference to the valid menu.
    # @since 2.0.0
    def validate(handle)
      AMS.validate_type(handle, Fixnum)
      return if AMS::C.is_menu?(handle)
      raise TypeError, "Handle is not a reference to a valid menu!", caller
    end

    # Determine whether handle is a reference to a valid menu.
    # @param [Fixnum] handle A menu handle to be tested.
    # @return [Boolean]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms647989(v=vs.85).aspx IsMenu
    # @since 2.0.0
    def is_valid?(handle)
      AMS::C.is_menu?(handle)
    end

    # Deactivate active context menu.
    # @return [Boolean] success
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms647637(v=vs.85).aspx EndMenu
    # @since 2.0.0
    def end
      AMC::C.end_menu()
    end

    # Get menu item count.
    # @param [Fixnum] handle A handle to the valid menu or sub-menu.
    # @return [Fixnum] count
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms647978(v=vs.85).aspx GetMenuItemCount
    # @since 2.0.0
    def get_item_count(handle)
      validate(handle)
      AMS::C.get_menu_item_count(handle)
    end

    # Get menu item id by item position.
    # @param [Fixnum] handle A handle to the valid menu or sub-menu.
    # @param [Fixnum] index
    # @return [Fixnum] Menu item identifier if successful or +-1+ if not.
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms647979(v=vs.85).aspx GetMenuItemID
    # @since 2.0.0
    def get_item_id(handle, index)
      validate(handle)
      AMS::C.get_menu_item_id(handle, index.to_i)
    end

    # Get menu sub-menu handle by sub-menu position.
    # @param [Fixnum] handle A handle to the valid menu or sub-menu.
    # @param [Fixnum] index
    # @return [Fixnum, nil] A handle to sub-menu if successful or +nil+ if not.
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms647984(v=vs.85).aspx GetSubMenu
    # @since 2.0.0
    def get_sub_menu(handle, index)
      validate(handle)
      AMS::C.get_sub_menu(handle, index.to_i)
    end

    # Get menu item string by item position.
    # @param [Fixnum] handle A handle to the valid menu or sub-menu.
    # @param [Fixnum] index
    # @return [String, nil] Menu item string if successful or +nil+ if not.
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms647980(v=vs.85).aspx GetMenuItemInfo
    # @since 2.0.0
    def get_menu_item_string_by_pos(handle, index)
      validate(handle)
      str = AMS::C.get_menu_item_string_by_pos(handle, index.to_i)
      str = str.unpack('C*').pack('U*') if str
      str
    end

    # Get menu item string by item identifier.
    # @param [Fixnum] handle A handle to the valid menu or sub-menu.
    # @param [Fixnum] id
    # @return [String, nil] Menu item string if successful or +nil+ if not.
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms647980(v=vs.85).aspx GetMenuItemInfo
    # @since 2.0.0
    def get_menu_item_string_by_id(handle, id)
      validate(handle)
      str = AMS::C.get_menu_item_string_by_id(handle, id.to_i)
      str = str.unpack('C*').pack('U*') if str
      str
    end

    # Set menu item string by item position.
    # @param [Fixnum] handle A handle to the valid menu or sub-menu.
    # @param [Fixnum] index
    # @param [String] string
    # @return [Boolean] success
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms648001(v=vs.85).aspx SetMenuItemInfo
    # @since 2.0.0
    def set_menu_item_string_by_pos(handle, index, string)
      validate(handle)
      AMS::C.set_menu_item_string_by_pos(handle, index.to_i, string.to_s.unpack('U*').pack('C*'))
    end

    # Set menu item string by item identifier.
    # @param [Fixnum] handle A handle to the valid menu or sub-menu.
    # @param [Fixnum] id
    # @param [String] string
    # @return [Boolean] success
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms648001(v=vs.85).aspx SetMenuItemInfo
    # @since 2.0.0
    def set_menu_item_string_by_id(handle, id, string)
      validate(handle)
      AMS::C.set_menu_item_string_by_id(handle, id.to_i, string.to_s.unpack('U*').pack('C*'))
    end

    # Get menu commands.
    # @param [Fixnum] handle A handle to the valid menu or sub-menu.
    # @param [String] cur_path Current menu path
    # @return [Hash{String => Fixnum}] { path => id, ... }
    # @since 2.0.0
    def get_commands(handle, cur_path = '')
      validate(handle)
      AMS::C.get_menu_commands(handle, cur_path.to_s)
    end

  end # class << self
end # module AMS::Menu
