module AMS::MIDI

  # @private
  module C
    if defined?(AMS::FFI)
      extend AMS::FFI::Library
      ffi_lib 'winmm.dll'

      callback :midi_out_proc, [:int, :uint, :pointer, :uint, :uint], :void

      attach_function :midiOutOpen, [:pointer, :int, :midi_out_proc, :pointer, :long], :int
      attach_function :midiOutClose, [:int], :int
      attach_function :midiOutReset, [:int], :int
      attach_function :midiOutShortMsg, [:int, :uint], :int
      attach_function :midiOutGetVolume, [:int, :pointer], :int
      attach_function :midiOutSetVolume, [:int, :uint], :int
      attach_function :midiOutGetNumDevs, [], :uint
    end
  end # module AMS::C

  # @!visibility private
  @handle = nil
  # @!visibility private
  @notes = {}
  # @!visibility private
  @timer = nil
  # @!visibility private
  MIDI_OUT_PROC = Proc.new { |hmo, msg, inst, param1, param2|
  }
  # @!visibility private
  NOTES_UPDATE_PROC = Proc.new {
    model = ::Sketchup.active_model
    view = model.active_view
    cam = view.camera
    tra = Geom::Transformation.new(cam.xaxis, cam.zaxis, cam.yaxis, cam.eye)
    @notes.each { |ch, d|
      pos = d[:pos].transform(tra.inverse)
      dist = ORIGIN.distance(pos)
      if dist > d[:range]
        change_channel_volume(ch, 0)
        change_channel_stereo_pan(ch, 64)
        next
      end
      h = Math.sqrt(pos.x**2 + pos.y**2)
      vol = ((d[:range]-dist).to_f * 127 / d[:range]).round
      if h.zero?
        change_channel_volume(ch, vol)
        change_channel_stereo_pan(ch, 64)
        next
      end
      angle = Math.acos(pos.x/h.to_f)
      ratio = 127 - (angle*127/Math::PI).round
      change_channel_volume(ch, vol)
      change_channel_stereo_pan(ch, ratio)
    }
  }

  class << self

    # Open device.
    # @return [Boolean] success
    # @since 2.0.0
    def open_device
      return false if @handle
      @handle = AMS::FFI::MemoryPointer.new(AMS::FFI.type_size(:int))
      C.midiOutOpen(@handle, 0, nil, nil, 0) == 0
    end

    # Close device.
    # @return [Boolean] success
    # @since 2.0.0
    def close_device
      return false unless @handle
      handle = @handle
      @handle = nil
      C.midiOutClose(handle.read_int) == 0
    end

    # Get volume of both channels.
    # @return [Array<Numeric>] +[left, right]+
    #   Each value is between 0.00 and 1.00.
    # @since 2.0.0
    def get_volume
      open_device
      volume = FFI::MemoryPointer.new(FFI.type_size(:ulong))
      C.midiOutGetVolume(@handle.read_int, volume)
      left = volume.read_ulong & 0xFFFF
      right = volume.read_ulong >> 16
      lr = (left*100.0/0xFFFF).round * 0.01
      rr = (right*100.0/0xFFFF).round * 0.01
      [lr, rr]
    end

    # @overload set_volume(left, right)
    #   Set volume of each channel.
    #   @param [Numeric] left This value is clamped between 0.0 and 1.0.
    #   @param [Numeric] right This value is clamped between 0.0 and 1.0.
    #   @return [Boolean] success
    #   @since 2.0.0
    # @overload set_volume(vol)
    #   Apply same volume to both channels.
    #   @param [Numeric] vol This value is clamped between 0.0 and 1.0.
    #   @return [Boolean] success
    #   @since 2.0.0
    def set_volume(*args)
      if args.size.between?(1,2)
        open_device
        left = AMS.clamp(args[0], 0, 1)
        right = (args.size == 1) ? left : AMS.clamp(args[1], 0, 1)
        r = (right*0xFFFF).to_i
        l = (left*0xFFFF).to_i
        C.midiOutSetVolume(@handle.read_int, r<<16|l) == 0
      else
        raise ArgumentError, "Expected 1 or 2 parameters, but got #{args.size}."
      end
    end

    # Stop all notes.
    # @return [Boolean] success
    # @since 2.0.0
    def reset
      open_device
      C.midiOutReset(@handle.read_int) == 0
      @notes.clear
      AMS::Timer.stop(@timer) if @timer
      @timer = nil
    end

    # Play a midi note.
    # @param [Fixnum] instrument between 0 and 127.
    # @param [Fixnum] note between 0 and 127. Each instrument has a maximum of
    #   127 notes.
    # @param [Fixnum] channel between 0 and 15. Each note has a maximum of 16
    #   channels. To play a variety of same notes at the same time, you must
    #   set an unused channel.
    # @param [Fixnum] volume between 0 and 127.
    # @param [Numeric, NilClass] duration Note duration in seconds. Pass +nil+
    #   to have sound play continuously.
    # @return [Boolean] success
    # @see http://wiki.fourthwoods.com/midi_file_format#general_midi_instrument_patch_map General MIDI Instrument Patch Map
    # @since 2.0.0
    def play_note(instrument, note, channel = 0, volume = 127, duration = nil)
      open_device
      instrument = AMS.clamp(instrument.to_i, 0, 127)
      note = AMS.clamp(note.to_i, 0, 127)
      channel = AMS.clamp(channel.to_i, 0, 15)
      volume = AMS.clamp(volume.to_i, 0, 127)
      # Set program or instrument.
      msg = instrument << 8 | (0xC0 | channel)
      C.midiOutShortMsg( @handle.read_int, msg )
      # Add timer to stop the note.
      if duration.is_a?(Numeric)
        AMS::Timer.start(duration, false){
          stop_note(instrument, note, channel)
        }
      end
      # Play note.
      h = volume
      l =  note << 8 | (0x90 | channel)
      msg = h << 16 | l
      C.midiOutShortMsg( @handle.read_int, msg ) == 0
    end

    # Stop a midi note.
    # @param [Fixnum] instrument between 0 and 127.
    # @param [Fixnum] note between 0 and 127.
    # @param [Fixnum] channel between 0 and 15.
    # @return [Boolean] success
    # @since 2.0.0
    def stop_note(instrument, note, channel = 0)
      open_device
      instrument = AMS.clamp(instrument.to_i, 0, 127)
      note = AMS.clamp(note.to_i, 0, 127)
      channel = AMS.clamp(channel.to_i, 0, 15)
      # Update 3d notes queue.
      @notes.keys.each { |ch|
        next if ch != channel
        next if @notes[ch][:instrument] != instrument
        next if @notes[ch][:note] != note
        @notes.delete(ch)
      }
      if @notes.empty? and @timer
        AMS::Timer.stop(@timer)
        @timer = nil
      end
      # Set program or instrument.
      msg = instrument << 8 | (0xC0 | channel)
      C.midiOutShortMsg( @handle.read_int, msg )
      # Stop note.
      msg = note << 8 | (0x80 | channel)
      C.midiOutShortMsg( @handle.read_int, msg ) == 0
    end

    # Set note controller.
    # @param [Fixnum] channel between 0 and 15.
    # @param [Fixnum] cnum between 0 and 127. Controller number.
    # @param [Fixnum] cval between 0 and 127. Controller value.
    # @return [Boolean] success
    # @see http://wiki.fourthwoods.com/midi_file_format#controller_codes MIDI Controller Codes
    # @since 2.0.0
    def change_channel_controller(channel, cnum, cval)
      open_device
      channel = AMS.clamp(channel.to_i, 0, 15)
      cnum = AMS.clamp(cnum.to_i, 0, 127)
      cval = AMS.clamp(cval.to_i, 0, 127)
      # Change note controller.
      h = cval
      l =  cnum << 8 | (0xB0 | channel)
      msg = h << 16 | l
      C.midiOutShortMsg( @handle.read_int, msg ) == 0
    end

    # Set channel volume.
    # @param [Fixnum] channel between 0 and 15.
    # @param [Fixnum] volume between 0 and 127. Soft to loud.
    # @return [Boolean] success
    # @since 2.0.0
    def change_channel_volume(channel, volume = 127)
      change_channel_controller(channel, 0x07, volume)
    end

    # Distribute channel volume between left and right speakers.
    # @param [Fixnum] channel between 0 and 15.
    # @param [Fixnum] pan between 0 and 127.
    #   0 is far left, 64 is center, 127 is far right.
    # @return [Boolean] success
    # @since 2.0.0
    def change_channel_stereo_pan(channel, pan = 64)
      change_channel_controller(channel, 0x0A, pan)
    end

    # Set channel expression. This is used for dynamics within a single track.
    # @param [Fixnum] channel between 0 and 15.
    # @param [Fixnum] expression between 0 and 127.
    # @return [Boolean] success
    # @since 2.0.0
    def change_channel_expression(channel, expression = 127)
      change_channel_controller(channel, 0x0B, expression)
    end

    # Control channel pedal. Message must be sent prior to the note it
    # effects.
    # @param [Fixnum] channel between 0 and 15.
    # @param [Boolean] state +true+ for pedal down, +false+ for pedal up.
    # @return [Boolean] success
    # @since 2.0.0
    def channel_sustain_pedal(channel, state = true)
      change_channel_controller(channel, 0x40, state ? 127 : 0)
    end

    # Reset channel controllers.
    # @param [Fixnum] channel between 0 and 15.
    # @return [Boolean] success
    # @since 2.0.0
    def reset_channel_controllers(channel)
      change_channel_controller(channel, 0x79, 0)
    end

    # Play a midi note at position.
    # @param [Geom::Point3d] pos Note position.
    # @param [Numeric] range Note hearing radius in inches.
    # @param [Fixnum] instrument between 0 and 127.
    # @param [Fixnum] note between 0 and 127. Each instrument has a maximum of
    #   127 notes.
    # @param [Fixnum] channel between 0 and 15. Each note has a maximum of 16
    #   channels. To play a variety of same notes at the same time, you must
    #   set an unused channel.
    # @param [Fixnum] max_volume between 0 and 127.
    # @param [Numeric, NilClass] duration Note duration in seconds. Pass +nil+
    #   to have sound play continuously.
    # @return [Boolean] success
    # @see http://wiki.fourthwoods.com/midi_file_format#general_midi_instrument_patch_map General MIDI Instrument Patch Map
    # @since 2.0.0
    def play_3d_note(pos, range, instrument, note, channel = 0, max_volume = 127, duration = nil)
      pos = Geom::Point3d.new(pos.to_a)
      range = range.to_f
      return false if range < 1
      instrument = AMS.clamp(instrument.to_i, 0, 127)
      note = AMS.clamp(note.to_i, 0, 127)
      channel = AMS.clamp(channel.to_i, 0, 15)
      max_volume = AMS.clamp(max_volume.to_i, 0, 127)
      if @notes[channel]
        stop_note(@notes[channel][:instrument], @notes[channel][:note], channel)
      end
      res = play_note(instrument, note, channel, max_volume, duration)
      return false unless res
      @notes[channel] = {
        :pos => pos,
        :range => range,
        :instrument => instrument,
        :note => note,
        :max_volume => max_volume
      }
      unless @timer
        @timer = AMS::Timer.start(1/20.0, true){
          NOTES_UPDATE_PROC.call
        }
      end
      true
    end

    # Change 3d note position.
    # @param [Fixnum] channel between 0 and 15.
    # @param [Geom::Point3d] pos Note position.
    # @return [Boolean] success
    # @since 2.0.0
    def set_3d_note_position(channel, pos)
      return false unless @notes[channel]
      @notes[channel][:pos] = Geom::Point3d.new(pos.to_a)
      true
    end

  end # class << self
end # module AMS::MIDI
