module AMS::Window
  class << self

    # Verify that handle is a reference to a valid window.
    # @param [Fixnum] handle
    # @return [void]
    # @raise [TypeError] if handle is not a reference to the valid window.
    # @since 2.0.0
    def validate(handle)
      AMS.validate_type(handle, Fixnum)
      return if AMS::C.is_window?(handle)
      raise TypeError, "Handle is not a reference to a valid window!", caller
    end

    # Determine whether handle is a reference to a valid window.
    # @param [Fixnum] handle A window handle to be tested.
    # @return [Boolean]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633528(v=vs.85).aspx IsWindow
    # @since 2.0.0
    def is_valid?(handle)
      AMS::C.is_window?(handle)
    end

    # Determine whether handle is a reference to an active window; the window with
    # which the user is currently working.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Boolean]
    # @since 2.0.0
    def is_active?(handle)
      validate(handle)
      AMS::C.is_window_active?(handle)
    end

    # Determine whether handle is a reference to a visible window.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Boolean]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633530(v=vs.85).aspx IsWindowVisible
    # @since 2.0.0
    def is_visible?(handle)
      validate(handle)
      AMS::C.is_window_visible?(handle)
    end

    # Determine whether handle is a reference to a maximized window.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Boolean]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633531(v=vs.85).aspx IsZoomed
    # @since 2.0.0
    def is_maximized?(handle)
      validate(handle)
      AMS::C.is_window_maximized?(handle)
    end

    # Determine whether handle is a reference to a minimized window.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Boolean]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633527(v=vs.85).aspx IsIconic
    # @since 2.0.0
    def is_minimized?(handle)
      validate(handle)
      AMS::C.is_window_minimized?(handle)
    end

    # Determine whether handle is a reference to a restored window; a window that
    # is not maximized or minimized.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Boolean]
    # @since 2.0.0
    def is_restored?(handle)
      validate(handle)
      (!AMS::C.is_window_minimized?(handle) && !AMS::C.is_window_minimized?(handle))
    end

    # Determine whether handle is a reference to a child window.
    # @param [Fixnum] parent_handle
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Boolean]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633524(v=vs.85).aspx IsChild
    # @since 2.0.0
    def is_child?(parent_handle, handle)
      validate(parent_handle)
      validate(handle)
      AMS::C.is_window_child?(parent_handle, handle)
    end

    # Set window show state.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Fixnum] state
    # @return [Fixnum] Previous show state.
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633548(v=vs.85).aspx ShowWindow
    # @since 2.0.0
    def show(handle, state)
      validate(handle)
      AMS.validate_type(state, Numeric)
      AMS::C.show_window(handle, state)
    end

    # Get active window.
    # @return [Fixnum] A handle to the active window.
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms646292(v=vs.85).aspx GetActiveWindow
    # @since 2.0.0
    def get_active
      AMS::C.get_active_window()
    end

    # Set active window.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Fixnum] A handle to the original active window.
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms646311(v=vs.85).aspx SetActiveWindow
    # @since 2.0.0
    def set_active(handle)
      validate(handle)
      AMS::C.set_active_window(handle)
    end

    # Get window's parent window.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Fixnum] If the window is a child window, the return value is a
    #   handle to the parent window. If the window is a top-level window with
    #   the WS_POPUP style, the return value is a handle to the owner window.
    #   Otherwise, the return value is zero.
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633510(v=vs.85).aspx GetParent
    # @since 2.0.0
    def get_parent(handle)
      validate(handle)
      AMS::C.get_parent_window(handle)
    end

    # Set window's parent window.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Fixnum] parent_handle A handle to the valid parent window.
    # @return [Fixnum] If the functions succeeds, the return values is a handle
    #   to the previous parent window. Otherwise, the return value is zero.
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633541(v=vs.85).aspx SetParent
    # @since 2.0.0
    def set_parent(handle, parent_handle)
      validate(handle)
      validate(parent_handle)
      AMS::C.set_parent_window(handle, parent_handle)
    end

    # Bring window to the top of the Z order. If the window is a top-level
    # window, it is activated. If the window is a child window, the top-level
    # parent window associated with the child window is activated.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Boolean] success
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms632673(v=vs.85).aspx BringWindowToTop
    # @since 2.0.0
    def bring_window_to_top(handle)
      validate(handle)
      AMS::C.bring_window_to_top(handle)
    end

    # Get window class name.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [String]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633582(v=vs.85).aspx GetClassName
    # @since 2.0.0
    def get_class_name(handle)
      validate(handle)
      AMS::C.get_window_class_name(handle).unpack('C*').pack('U*')
    end

    # Get window thread ID.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Fixnum]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633522(v=vs.85).aspx GetWindowThreadProcessId
    # @since 2.0.0
    def get_thread_id(handle)
      validate(handle)
      AMS::C.get_window_thread_id(handle)
    end

    # Get window process ID.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Fixnum]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633522(v=vs.85).aspx GetWindowThreadProcessId
    # @since 2.0.0
    def get_process_id(handle)
      validate(handle)
      AMS::C.get_window_process_id(handle)
    end

    # Get window long.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Fixnum] index
    # @return [Fixnum] Window long.
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633584(v=vs.85).aspx GetWindowLong
    # @since 2.0.0
    def get_long(handle, index)
      validate(handle)
      AMS::C.get_window_long(handle, index)
    end

    # Set window long.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Fixnum] index
    # @param [Fixnum] long
    # @return [Fixnum] Previous window long.
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633591(v=vs.85).aspx SetWindowLong
    # @since 2.0.0
    def set_long(handle, index, long)
      validate(handle)
      AMS.validate_type(index, Numeric)
      AMS.validate_type(long, Numeric)
      AMS::C.set_window_long(handle, index.to_i, long.to_i)
    end

    # Get window menu.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Fixnum] A handle to the menu. If the specified window has no
    #   menu, the return value is zero.
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms647640(v=vs.85).aspx GetMenu
    # @since 2.0.0
    def get_menu(handle)
      validate(handle)
      AMS::C.get_window_menu(handle)
    end

    # Set window menu.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Fixnum, nil] menu_handle A handle to the new menu. If this
    #   parameter is +nil+, the window's current menu is removed.
    # @return [Boolean] success
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms647995(v=vs.85).aspx SetMenu
    # @since 2.0.0
    def set_menu(handle, menu_handle)
      validate(handle)
      AMS.validate_type(menu_handle, Numeric) if menu_handle != nil
      AMS::C.set_window_menu(handle, menu_handle.to_i)
    end

    # Get window caption.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [String]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633520(v=vs.85).aspx GetWindowText
    # @since 2.0.0
    def get_text(handle)
      validate(handle)
      AMS::C.get_window_text(handle).unpack('C*').pack('U*')
    end

    # Set window caption.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [String] caption
    # @return [Boolean] success
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633546(v=vs.85).aspx SetWindowText
    # @since 2.0.0
    def set_text(handle, caption)
      validate(handle)
      AMS.validate_type(caption, String)
      AMS::C.set_window_text(handle, caption.unpack('U*').pack('C*'))
    end

    # Get upper-left and lower-right coordinates of the window in screen
    # coordinates, relative to the upper-left corner of the screen.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Array<Fixnum>] An array of four numeric values, representing
    #   upper-left and lower-right coordinates: [x1,y1, x2,y2].
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633519(v=vs.85).aspx GetWindowRect
    # @since 2.0.0
    def get_rect(handle)
      validate(handle)
      AMS::C.get_window_rect(handle)
    end

    # Get upper-left and lower-right coordinates of the window's client area in
    # client coordinates, relative to the upper-left corner of the window's
    # client area. Because coordinates are relative to the upper-left corner of
    # the client area, the coordinates of the upper-left corner are (0,0).
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Array<Fixnum>] An array of four numeric values, representing
    #   upper-left and lower-right coordinates: [x1,y1, x2,y2].
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633503(v=vs.85).aspx GetClientRect
    # @since 2.0.0
    def get_client_rect(handle)
      validate(handle)
      AMS::C.get_client_rect(handle)
    end

    # Adjust window rect.
    # @param [Array<Fixnum>] rect
    # @param [Fixnum] style
    # @param [Fixnum] style_ex
    # @param [Boolean] b_menu
    # @return [Array<Fixnum>] New rect: [x1,y1, x2,y2].
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms632667(v=vs.85).aspx AdjustWindowRectEx
    # @since 2.0.0
    def adjust_rect(rect, style, style_ex, b_menu)
      AMS.validate_type(rect, Enumerable)
      AMS.validate_type(style, Numeric)
      AMS.validate_type(style_ex, Numeric)
      AMS.validate_type(b_menu, TrueClass, FalseClass)
      AMS::C.adjust_window_rect(rect, style.to_i, style_ex.to_i, b_menu)
    end

    # Get window placement.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Array] An array of 6 objects representing window placement:
    #   [length, flags, show_cmd, min_pt, max_pt, rect].
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633518(v=vs.85).aspx GetWindowPlacement
    # @since 2.0.0
    def get_placement(handle)
      validate(handle)
      AMS::C.get_window_placement(handle)
    end

    # Set Window Placement.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Fixnum] flags
    # @param [Fixnum] show_cmd
    # @param [Array<Fixnum>] min_pt (x,y)
    # @param [Array<Fixnum>] max_pt (x,y)
    # @param [Array<Fixnum>] rect (x1,y1, x2,y2)
    # @return [Boolean] success
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633544(v=vs.85).aspx SetWindowPlacement
    # @since 2.0.0
    def set_placement(handle, flags, show_cmd, min_pt, max_pt, rect)
      validate(handle)
      AMS.validate_type(flags, Numeric)
      AMS.validate_type(show_cmd, Numeric)
      AMS.validate_type(min_pt, Enumerable)
      AMS.validate_type(max_pt, Enumerable)
      AMS.validate_type(rect, Enumerable)
      AMS::C.set_window_placement(handle, flags.to_i, show_cmd.to_i, min_pt, max_pt, rect)
    end

    # Lock window update.
    # @param [Fixnum, nil] handle A handle to the valid window to be locked.
    #   If this parameter is +nil+, the window is unlocked.
    # @return [Boolean] success
    # @note This method is usually called before and after setting window
    #   styles, followed by {set_pos} to update setting.
    # @example Remove caption from the active window.
    #   handle = AMS::Window.get_active
    #   long = AMS::Window.get_long(handle, -16)
    #   AMS::Window.lock_update(handle)
    #   AMS::Window.set_long(handle, -16, long & ~0x00C00000)
    #   AMS::Window.lock_update(nil)
    #   AMS::Window.set_pos(handle, 0, 0, 0, 0, 0, 0x0277)
    # @example Set caption to the active window.
    #   handle = AMS::Window.get_active
    #   long = AMS::Window.get_long(handle, -16)
    #   AMS::Window.lock_update(handle)
    #   AMS::Window.set_long(handle, -16, long | 0x00C00000)
    #   AMS::Window.lock_update(nil)
    #   AMS::Window.set_pos(handle, 0, 0, 0, 0, 0, 0x0277)
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/dd145034(v=vs.85).aspx LockWindowUpdate
    # @since 2.0.0
    def lock_update(handle)
      validate(handle) if handle != nil
      AMS::C.lock_window_update(handle)
    end

    # Set window position.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Fixnum] handle_insert_after
    # @param [Fixnum] x
    # @param [Fixnum] y
    # @param [Fixnum] cx
    # @param [Fixnum] cy
    # @param [Fixnum] flags
    # @return [Boolean] success
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633545(v=vs.85).aspx SetWindowPos
    # @since 2.0.0
    def set_pos(handle, handle_insert_after, x, y, cx, cy, flags)
      validate(handle)
      AMS.validate_type(handle_insert_after, Fixnum)
      AMS.validate_type(x, Numeric)
      AMS.validate_type(y, Numeric)
      AMS.validate_type(cx, Numeric)
      AMS.validate_type(cy, Numeric)
      AMS.validate_type(flags, Numeric)
      AMS::C.set_window_pos(handle, handle_insert_after, x, y, cx, cy, flags)
    end

    # Move window.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Fixnum] x
    # @param [Fixnum] y
    # @param [Fixnum] width
    # @param [Fixnum] height
    # @param [Boolean] b_repaint
    # @return [Boolean] success
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633534(v=vs.85).aspx MoveWindow
    # @since 2.0.0
    def move(handle, x, y, width, height, b_repaint = true)
      validate(handle)
      AMS.validate_type(x, Numeric)
      AMS.validate_type(y, Numeric)
      AMS.validate_type(width, Numeric)
      AMS.validate_type(height, Numeric)
      AMS.validate_type(b_repaint, TrueClass, FalseClass)
      AMS::C.move_window(handle, x, y, width, height, b_repaint)
    end

    # Refresh window.
    # @param [Fixnum] handle A handle to the valid window.
    # @since 2.0.0
    def refresh(handle)
      validate(handle)
      AMS::C.refresh_window(handle)
    end

    # Close window. This behaves the same as clicking the 'X' button.
    # @param [Fixnum] handle A handle to the valid window.
    # @since 2.0.0
    def close(handle)
      validate(handle)
      AMS::C.close_window(handle)
    end

    # Get window origin.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Array<Fixnum>] [x,y]
    # @since 2.0.0
    def get_origin(handle)
      get_rect(handle)[0,2]
    end

    # Set window origin.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Fixnum] x
    # @param [Fixnum] y
    # @param [Boolean] b_activate Whether to activate the window.
    # @return [Boolean] success
    # @since 2.0.0
    def set_origin(handle, x, y, b_activate = true)
      set_pos(handle, 0, x, y, 0, 0, 0x0261 | (b_activate ? 0 : 0x0014))
    end

    # Get window size.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Array<Fixnum>] [w,h]
    # @since 2.0.0
    def get_size(handle)
      r = get_rect(handle)
      [r[2]-[0], r[3]-r[1]]
    end

    # Set window size.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Fixnum] w
    # @param [Fixnum] h
    # @param [Boolean] b_activate Whether to activate the window.
    # @return [Boolean] success
    # @since 2.0.0
    def set_size(handle, w, h, b_activate = true)
      set_pos(handle, 0, 0, 0, w, h, 0x0262 | (b_activate ? 0 : 0x0014))
    end

    # Set upper-left and lower-right coordinates of the window.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Fixnum] x1
    # @param [Fixnum] y1
    # @param [Fixnum] x2
    # @param [Fixnum] y2
    # @param [Boolean] b_activate Whether to activate the window.
    # @return [Boolean] success
    # @since 2.0.0
    def set_rect(handle, x1, y1, x2, y2, b_activate = true)
      set_pos(handle, 0, x1, y1, x2-x1, y2-y1, 0x0260 | (b_activate ? 0 : 0x0014))
    end

    # Determine whether window is resizeable.
    # @return [Boolean]
    # @since 2.0.0
    def is_resizeable?(handle)
      style = get_long(handle, -16)
      (style | 0x00040000 ) == style # WS_SIZEBOX
    end

    # Set window resizeable.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Boolean] b_state
    # @param [Boolean] b_activate Whether to activate the window.
    # @return [Boolean] success
    # @since 2.0.0
    def set_resizeable(handle, b_state, b_activate = true)
      b_state = b_state ? true : false
      return false if b_state == is_resizeable?(handle)
      style = get_long(handle, -16)
      new_style = b_state ? style | 0x00040000 : style & ~0x00040000
      lock_update(handle)
      set_long(handle, -16, new_style)
      lock_update(nil)
      set_pos(handle, 0, 0, 0, 0, 0, 0x0267 | (b_activate ? 0 : 0x0014))
      true
    end

    # Enable/Disable keyboard and mouse input to the specified window.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Boolean] b_state
    # @return [Boolean] success
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms646291(v=vs.85).aspx EnableWindow
    # @since 2.0.0
    def enable_input(handle, b_state)
      validate(handle)
      AMS::C.enable_window(handle, b_state)
    end

    # Determine whether keyboard and mouse input is enabled to the specified
    # window.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Boolean]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms646303(v=vs.85).aspx IsWindowEnabled
    # @since 2.0.0
    def is_input_enabled?(handle)
      validate(handle)
      AMS::C.is_window_enabled?(handle)
    end

    # Send Message to the specified window.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Fixnum, Bignum] message
    # @param [Fixnum, Bignum] wParam
    # @param [Fixnum, Bignum] lParam
    # @return [nil]
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms644950(v=vs.85).aspx SendMessage
    # @since 2.0.0
    def send_message(handle, message, wParam, lParam)
      validate(handle)
      AMS::C.send_message(handle, message.to_i, wParam.to_i, lParam.to_i)
    end

    # Modify window icon.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [String] path Path to whether the icon is located.
    # @return [Boolean] success
    # @since 2.0.0
    def set_icon(handle, path)
      validate(handle)
      path = path.gsub("/", "\\").unpack('U*').pack('C*')
      AMS::C.set_window_icon(handle, path)
    end

    # Update menu bar of the specified window.
    # @param [Fixnum] handle A handle to the valid window.
    # @return [Boolean] success
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms647633(v=vs.85).aspx DrawMenuBar
    # @since 2.0.0
    def draw_menu_bar(handle)
      validate(handle)
      AMS::C.draw_menu_bar(handle)
    end

    # Set layered window attributes.
    # @param [Fixnum] handle A handle to the valid window.
    # @param [Fixnum, Bignum] color
    # @param [Fixnum] opacity
    # @param [Fixnum] flags
    # @return [Boolean] success
    # @see http://msdn.microsoft.com/en-us/library/windows/desktop/ms633540(v=vs.85).aspx SetLayeredWindowAttributes
    # @since 2.0.0
    def set_layered_attributes(handle, color, opacity, flags)
      validate(handle)
      AMS::C.set_layered_window_attributes(handle, color.to_i, opacity.to_i, flags.to_i)
    end

  end # class << self
end # AMS::Window
